@@ -15,7 +15,7 @@
 !  a) You must cause the modified files to carry prominent notices
 !     stating that you changed the files and the date of any change.
 !
-!  Copyright (C) 2005-2015 Daniel Price. All rights reserved.
+!  Copyright (C) 2005-2018 Daniel Price. All rights reserved.
 !  Contact: daniel.price@monash.edu
 !
 !-----------------------------------------------------------------
@@ -26,7 +26,7 @@
 !-----------------------------------------------------------------
 module calcquantities
  use labels, only:lenlabel,lenunitslabel
- use params, only:maxplot
+ use params, only:maxplot,maxhdr
  implicit none
  public :: calc_quantities,setup_calculated_quantities
  public :: calc_quantities_use_x0,get_calc_data_dependencies
@@ -148,17 +148,17 @@ subroutine add_calculated_quantities(istart,iend,ncalc,printhelp,incolumn,verbos
  use prompting,     only:prompt
  use fparser,       only:checkf
  use settings_data, only:ncolumns,iRescale,required
- use labels,        only:shortstring
+ use labels,        only:shortstring,headertags,count_non_blank
  implicit none
  integer, intent(in)  :: istart,iend
  integer, intent(out) :: ncalc
  logical, intent(in)  :: printhelp
  integer, dimension(maxcalc), intent(in) :: incolumn
  logical, intent(in)  :: verbose
- integer :: i,j,ntries,ierr,iequal,nvars
+ integer :: i,j,ntries,ierr,iequal,nvars,nhdr
  logical :: iask
  character(len=120) :: string
- character(len=lenvars), dimension(maxplot+nextravars) :: vars
+ character(len=lenvars), dimension(maxplot+nextravars+maxhdr) :: vars
 
  i = istart + 1
  ntries = 0
@@ -173,9 +173,11 @@ subroutine add_calculated_quantities(istart,iend,ncalc,printhelp,incolumn,verbos
 
  if (printhelp) then
     print "(/,a)",' Specify a function to calculate from the data '
-    print "(10(a))",' Valid variables are the column labels',(', '''//trim(extravars(j))//'''',j=1,nextravars-1),&
-                ' and '''//trim(extravars(nextravars))//''''
-    print "(a)",' Spaces, escape sequences (\d), arithmetic operators and units labels'
+    print "(10(a))",' Valid variables are column labels',(', '''//trim(extravars(j))//'''',j=1,nextravars),&
+                ' and header variables:'
+    nhdr = count_non_blank(headertags)
+    print "(43(2x,6(a),/))",headertags(1:nhdr)
+    print "(/,a)",' Spaces, escape sequences (\d), arithmetic operators and units labels'
     print "(a)",' are removed from variable names. Note that previously calculated'
     print "(a)",' quantities can be used in subsequent calculations.'
  endif
@@ -304,16 +306,16 @@ end subroutine splitstring
 subroutine print_example_quantities(verbose,ncalc)
  use labels,        only:label,unitslabel,shortlabel,lenlabel,irho,iutherm,iBfirst,&
                          ix,icv,idivB,ih,iradenergy,iamvec,labelvec,idustfrac,&
-                         idustfracsum,ideltav,ivx
- use settings_data, only:ncolumns,ndim,icoordsnew,ndimV,ndusttypes,idustfrac_plot
+                         ideltav,ivx
+ use settings_data, only:ncolumns,ndim,icoordsnew,ndimV
  use geometry,      only:labelcoord
+ use asciiutils,    only:append_number,find_repeated_tags
  implicit none
  logical :: verbose
  integer, intent(inout), optional :: ncalc
  logical :: prefill
- character(len=lenlabel) :: string,temp,labelprev
- character(len=lenlabel) :: ldfrac,ldfake,ldfracsum,idust_string
- integer :: i,j,ivecstart,ierr,ilen
+ character(len=lenlabel) :: string,labelprev,ldfracsum
+ integer :: i,j,ivecstart,ierr,ilen,idustfrac1,ndusttypes
  logical :: gotpmag,gotpressure
 
  gotpmag = .false.
@@ -329,17 +331,6 @@ subroutine print_example_quantities(verbose,ncalc)
     endif
  endif
 
- if (idustfrac > 0) then
-    if (idustfracsum == 0) then
-       !--one dust phase
-       ldfracsum = shortlabel(label(idustfrac),unitslabel(idustfrac))
-    else
-       !--multiple dust phases
-       if (verbose) print*,' Warning! Density refers to total density: rhogas+rhodust'
-       ldfracsum = shortlabel(label(idustfracsum),unitslabel(idustfracsum))
-    endif
- endif
-
  !--radius
  string = ' '
  if (ndim.gt.0 .and. icoordsnew.eq.1 .and. ncolumns.ge.ndim) then
@@ -385,11 +376,35 @@ subroutine print_example_quantities(verbose,ncalc)
        print "(11x,a)",trim(string)
     endif
  endif
-
+ !
+ !--total dust fraction if multiple dust phases
+ !
+ ldfracsum = ' '
+ if (idustfrac == 0) then
+    ! find number of dust species from how many times the label "dustfrac"
+    ! is repeated. For gas-only calculations, this will give ndusttypes=0
+    call find_repeated_tags('dustfrac',ncolumns,label,idustfrac1,ndusttypes)
+    if (ndusttypes > 1) then
+       string = 'dustfrac = '//trim(shortlabel(label(idustfrac1),unitslabel(idustfrac1)))
+       do i=idustfrac1+1,idustfrac1+ndusttypes-1
+          string = trim(string)//'+'//(trim(shortlabel(label(i),unitslabel(i))))
+       enddo
+       ldfracsum = 'dustfrac'
+       if (prefill) then
+          ncalc = ncalc + 1
+          call splitstring(string,calclabel(ncalc),calcstring(ncalc))
+       else
+          print "(11x,a)",trim(string)
+       endif
+    endif
+ else
+    ndusttypes = 1
+    ldfracsum = shortlabel(label(idustfrac),unitslabel(idustfrac))
+ endif
  !
  !--one-fluid dust stuff
  !
- if (idustfrac.gt.0 .and. irho.gt.0) then
+ if (len_trim(ldfracsum) > 0 .and. irho.gt.0) then
     string = ' '
     !--gas density
     write(string,"(a)",iostat=ierr) '\rho_{g} = ' &
@@ -402,30 +417,34 @@ subroutine print_example_quantities(verbose,ncalc)
        calcunitslabel(ncalc) = unitslabel(irho)
     else
        print "(11x,a)",trim(string)
-       call splitstring(string,labelprev,temp)
     endif
-    !--dust density
-    do i = 1,ndusttypes
-       if (ndusttypes>1) then
-          write(idust_string,"(I10)") i
-          write(idust_string,"(a)") '{d,'//trim(adjustl(idust_string))//'}'
-          ldfrac = shortlabel(label(idustfracsum+i),unitslabel(idustfracsum+i))    
-          ldfake = shortlabel(label(idustfrac_plot),unitslabel(idustfrac))
-       else
-          write(idust_string,"(a)") '{d}'
-          ldfrac = shortlabel(label(idustfrac),unitslabel(idustfrac))    
-          ldfake = '1'
-       endif
-       write(string,"(a)",iostat=ierr) '\rho_'//trim(adjustl(idust_string))//' = ' &
-                       //trim(ldfrac)//'*'//trim(shortlabel(label(irho),unitslabel(irho)))
-       if (prefill) then
-          ncalc = ncalc + 1
-          call splitstring(string,calclabel(ncalc),calcstring(ncalc))
-          calcunitslabel(ncalc) = unitslabel(irho)
-       else
-          print "(11x,a)",trim(string)
-       endif
-    enddo
+    !--(total) dust density
+    write(string,"(a)",iostat=ierr) '\rho_{d} = ' &
+                    //trim(shortlabel(label(irho),unitslabel(irho))) &
+                    //'*'//trim(ldfracsum)
+    if (prefill) then
+       ncalc = ncalc + 1
+       call splitstring(string,calclabel(ncalc),calcstring(ncalc))
+       calcunitslabel(ncalc) = unitslabel(irho)
+    else
+       print "(11x,a)",trim(string)
+    endif
+    !--densities of each individual dust phase
+    if (ndusttypes > 1) then
+       do i = idustfrac1,idustfrac1+ndusttypes-1
+          string = '\rho_{d,'
+          call append_number(string,i-idustfrac1+1)
+          string = trim(string)//'} = ' &
+               //trim(shortlabel(label(irho),unitslabel(irho)))//'*'//trim(label(i))
+          if (prefill) then
+             ncalc = ncalc + 1
+             call splitstring(string,calclabel(ncalc),calcstring(ncalc))
+             calcunitslabel(ncalc) = unitslabel(irho)
+          else
+             print "(11x,a)",trim(string)
+          endif
+       enddo
+    endif
     !--dust-to-gas ratio
     write(string,"(a)",iostat=ierr) 'dust-to-gas ratio = ' &
                     //trim(ldfracsum)//'/(1. - '//trim(ldfracsum)//')'
@@ -454,7 +473,7 @@ subroutine print_example_quantities(verbose,ncalc)
                 call splitstring(string,calclabel(ncalc),calcstring(ncalc))
              else
                 print "(11x,a)",trim(string)
-             endif       
+             endif
           enddo
           !--dust velocities
           do i=1,ndimV
@@ -472,13 +491,13 @@ subroutine print_example_quantities(verbose,ncalc)
                 call splitstring(string,calclabel(ncalc),calcstring(ncalc))
              else
                 print "(11x,a)",trim(string)
-             endif       
+             endif
           enddo
        else
           ! Still needs to be implemented...
        endif
     endif
- endif 
+ endif
 
  !
  !--magnitudes of all vector quantities (only if cartesian coords are set)
@@ -517,7 +536,8 @@ subroutine print_example_quantities(verbose,ncalc)
     ilen = len_trim(string)
     if (ndimV.gt.1) then
        write(string(ilen+1:),"(a,a,a)",iostat=ierr) &
-            (' + '//trim(shortlabel(label(i),unitslabel(i)))//'**2',i=iBfirst+1,iBfirst+ndimV-1),')'
+            (' + '//trim(shortlabel(label(i),unitslabel(i))) &
+            //'**2',i=iBfirst+1,iBfirst+ndimV-1),')'
     else
        write(string(ilen+1:),"(a)",iostat=ierr) ')'
     endif
@@ -584,7 +604,7 @@ subroutine print_example_quantities(verbose,ncalc)
        print "(11x,a)",trim(string)
     endif
  endif
- 
+
  if (.not.prefill) print "(a)"
 
 end subroutine print_example_quantities
@@ -604,7 +624,7 @@ subroutine check_calculated_quantities(ncalcok,ncalctot,incolumn,verbose)
  integer, dimension(maxcalc), intent(out), optional :: incolumn
  logical, intent(in), optional :: verbose
  integer :: i,ierr,nvars,indexinactive
- character(len=lenvars), dimension(maxplot+nextravars) :: vars
+ character(len=lenvars), dimension(maxplot+nextravars+maxhdr) :: vars
  logical :: isverbose
 
  if (present(verbose)) then
@@ -671,7 +691,7 @@ subroutine get_calc_data_dependencies(required)
  use fparser,        only:checkf
  use labels,         only:label,shortlabel,shortstring
  logical, dimension(0:maxplot), intent(inout) :: required
- character(len=lenvars), dimension(maxplot+nextravars) :: vars
+ character(len=lenvars), dimension(maxplot+nextravars+maxhdr) :: vars
  integer, dimension(maxcalc) :: incolumn
  integer :: ncalcok,ncalctot,nvars,i,j
 
@@ -706,7 +726,7 @@ subroutine get_calc_data_dependencies(required)
        endif
     endif
  enddo
- 
+
 end subroutine get_calc_data_dependencies
 
 !-----------------------------------------------------------------
@@ -716,11 +736,10 @@ end subroutine get_calc_data_dependencies
 !-----------------------------------------------------------------
 subroutine calc_quantities(ifromstep,itostep,dontcalculate)
   use labels,         only:label,unitslabel,labelvec,iamvec,ix,ivx,shortstring, &
-                           irho,idustfracsum
-  use particle_data,  only:dat,npartoftype,gamma,time,maxpart,maxstep,maxcol,iamtype
+                           count_non_blank,headertags
+  use particle_data,  only:dat,npartoftype,gamma,time,headervals,maxpart,maxstep,maxcol,iamtype
   use settings_data,  only:ncolumns,ncalc,iRescale,xorigin,debugmode,ndim,required,iverbose, &
-                           icoords,icoordsnew,ipartialread,itracktype,itrackoffset,ndusttypes, &
-                           idustfrac_plot
+                           icoords,icoordsnew,ipartialread,itracktype,itrackoffset
   use mem_allocation, only:alloc
   use settings_units, only:units
   use fparser,        only:checkf,parsef,evalf,EvalerrMsg,EvalErrType,rn,initf,endf
@@ -731,13 +750,14 @@ subroutine calc_quantities(ifromstep,itostep,dontcalculate)
   implicit none
   integer, intent(in) :: ifromstep, itostep
   logical, intent(in), optional :: dontcalculate
-  integer :: i,j,ncolsnew,ierr,icalc,ntoti,nvars,ncalctot,nused,itrackpart,ndust,icolumn
+  integer :: i,j,ncolsnew,ierr,icalc,ntoti,nvars,ncalctot,nused,itrackpart
+  integer :: ndust,nhdr
   logical :: skip
 !  real, parameter :: mhonkb = 1.6733e-24/1.38e-16
 !  real, parameter :: radconst = 7.5646e-15
 !  real, parameter :: lightspeed = 3.e10   ! in cm/s (cgs)
-  real(kind=rn), dimension(maxplot+nextravars)          :: vals
-  character(len=lenvars), dimension(maxplot+nextravars) :: vars
+  real(kind=rn), dimension(maxplot+nextravars+maxhdr)          :: vals
+  character(len=lenvars), dimension(maxplot+nextravars+maxhdr) :: vars
   real, dimension(3) :: x0,v0
   real :: t1,t2
 
@@ -849,6 +869,10 @@ subroutine calc_quantities(ifromstep,itostep,dontcalculate)
               vals(ncolumns+icalc+2) = x0(1)
               vals(ncolumns+icalc+3) = x0(2)
               vals(ncolumns+icalc+4) = x0(3)
+              nhdr = count_non_blank(headertags)
+              do j=1,nhdr
+                 vals(ncolumns+icalc+4+j) = headervals(j,i)
+              enddo
               if (icoordsnew.ne.icoords .and. ndim.gt.0 .and. all(ix(1:ndim).gt.0)) then
                  !
                  !--if alternative coordinate system is in use, then we need to apply
@@ -857,34 +881,16 @@ subroutine calc_quantities(ifromstep,itostep,dontcalculate)
                  !
                  do j=1,ntoti
                     vals(1:ncolumns+icalc-1) = dat(j,1:ncolumns+icalc-1,i)
-                    call change_coords(vals(1:ncolumns+icalc-1),ncolumns+icalc-1,ndim,icoords,icoordsnew,x0(1:ndim),v0(1:ndim))
+                    call change_coords(vals(1:ncolumns+icalc-1),ncolumns+icalc-1,&
+                                       ndim,icoords,icoordsnew,x0(1:ndim),v0(1:ndim))
                     !--evaluate function with transformed values
-                    dat(j,ncolumns+icalc,i) = real(evalf(icalc,vals(1:ncolumns+icalc+nextravars-1)))
+                    dat(j,ncolumns+icalc,i) = real(evalf(icalc,vals(1:ncolumns+icalc+nextravars+nhdr-1)))
                  enddo
               else
-                 !!$omp parallel do default(none) private(j,vals,icolumn) shared(dat,i,icalc,ncolumns,ndusttypes,iamtype,idustfracsum,idustfrac_plot)
+                 !!$omp parallel do default(none) private(j,vals,icolumn) shared(dat,i,icalc,ncolumns)
                  do j=1,ntoti
-                    do icolumn = 1,ncolumns+icalc-1
-                       if (ndusttypes>1 .and. icolumn==irho) then
-                          !
-                          !--reverse the fake data calculations if needed in order to make
-                          !  the input equations in the calculated extra quantities consistent
-                          !  with the physical equations
-                          !
-                          select case(iamtype(j,i))
-                          case(1) !--gas
-                             vals(icolumn) = dat(j,icolumn,i)/(1.-dat(j,idustfracsum,i))
-                          case(2) !--dust
-                             vals(icolumn) = dat(j,icolumn,i)/dat(j,idustfrac_plot,i)
-                          case default !--other stuff
-                             vals(icolumn) = dat(j,icolumn,i)
-                          endselect
-                       else
-                          vals(icolumn) = dat(j,icolumn,i)
-                       endif
-                    enddo
-                    !vals(1:ncolumns+icalc-1) = dat(j,1:ncolumns+icalc-1,i)
-                    dat(j,ncolumns+icalc,i) = real(evalf(icalc,vals(1:ncolumns+icalc+nextravars-1)))
+                    vals(1:ncolumns+icalc-1) = dat(j,1:ncolumns+icalc-1,i)
+                    dat(j,ncolumns+icalc,i) = real(evalf(icalc,vals(1:ncolumns+icalc+nextravars+nhdr-1)))
                  enddo
                  !!$omp end parallel do
               endif
@@ -976,12 +982,12 @@ end subroutine identify_calculated_quantity
 !
 !-----------------------------------------------------------------
 subroutine get_variables(maxlabel,nvars,variables)
- use labels,         only:label,shortlabel,unitslabel
+ use labels,         only:label,shortlabel,unitslabel,headertags,count_non_blank
  implicit none
  integer,                        intent(in)  :: maxlabel
  integer,                        intent(out) :: nvars
  character(len=*), dimension(:), intent(out) :: variables
- integer :: i
+ integer :: i,nheader
  !
  !--can use column labels up to the previous quantity calculated
  !
@@ -994,6 +1000,11 @@ subroutine get_variables(maxlabel,nvars,variables)
  do i=1,nextravars
     variables(maxlabel+i) = trim(extravars(i))
  enddo
+ nheader = count_non_blank(headertags)
+ nvars = nvars + nheader
+ do i=1,nheader
+    variables(maxlabel+nextravars+i) = trim(headertags(i))
+ enddo
 
 end subroutine get_variables
 
