@@ -58,7 +58,7 @@ module oilonwaterread
 end module oilonwaterread
 
 
-subroutine read_data(rootname,indexstart,nstepsread)
+subroutine read_data(rootname,indexstart,ipos,nstepsread)
   use particle_data
   use params
   use settings_data, only:ndim,ndimV,ncolumns,ncalc
@@ -66,7 +66,7 @@ subroutine read_data(rootname,indexstart,nstepsread)
   use labels, only:ix,ivx,ih,irho,ipmass
   use oilonwaterread, only:udisti,umassi,utimei
   implicit none
-  integer, intent(in) :: indexstart
+  integer, intent(in) :: indexstart,ipos
   integer, intent(out) :: nstepsread
   character(len=*), intent(in) :: rootname
   integer, parameter :: maxptmass = 1000
