@@ -15,7 +15,7 @@
 !  a) You must cause the modified files to carry prominent notices
 !     stating that you changed the files and the date of any change.
 !
-!  Copyright (C) 2005-2018 Daniel Price. All rights reserved.
+!  Copyright (C) 2005-2019 Daniel Price. All rights reserved.
 !  Contact: daniel.price@monash.edu
 !
 !  The plotting API for SPLASH 2.0 was written by James Wetter
@@ -27,7 +27,7 @@ program splash
 !---------------------------------------------------------------------------------
 !
 !     SPLASH - a plotting utility for SPH data in 1, 2 and 3 dimensions
-!     Copyright (C) 2005-2018 Daniel Price
+!     Copyright (C) 2005-2019 Daniel Price
 !     daniel.price@monash.edu
 !
 !     --------------------------------------------------------------------------
@@ -48,6 +48,16 @@ program splash
 !
 !     -------------------------------------------------------------------------
 !     Version history/ Changelog:
+!     2.9.0  : (05/04/19)
+!             general header quantities are read and available in function parser;
+!             more robust label detection and parsing during ascii data read;
+!             splash to grid works in non-cartesian geometries; added flared and
+!             log-flared coordinate systems; Doppler shift colour bar; can customise
+!             line style and colour when plotting multiple exact solutions; seg faults fixed;
+!             better plot tiling decisions; disappearing arrows bug fix; Rafikov disc-
+!             planet exact solution added; atan2 implemented in function parser;
+!             various multigrain phantom read fixes (incl. seg faults); exact rendering
+!             implemented in 2D; libsplash implemented for use as Python splash backend
 !     2.8.0  : (06/04/18)
 !             360/4pi video mode added; automatically read labels from ascii file headers;
 !             nearest sensible unit (e.g. au or pc) used by default; cactus hdf5 data read;
@@ -380,12 +390,15 @@ program splash
   use analysis,           only:isanalysis
   use timestepping,       only:timestep_loop
   use settings_page,      only:interactive,device,nomenu
+  use settings_part,      only:initialise_coord_transforms
+  use settings_render,    only:icolours,rgbfile
+  use colours,            only:rgbtable,ncoltable,icustom
   implicit none
   integer :: i,ierr,nargs,ipickx,ipicky,irender,icontour,ivecplot
   logical :: ihavereadfilenames,evsplash,doconvert,useall,iexist,use_360
   character(len=120) :: string
   character(len=12)  :: convertformat
-  character(len=*), parameter :: version = 'v2.8.0 [6th April 2018]'
+  character(len=*), parameter :: version = 'v2.9.0 [5th Apr 2019]'
 
   !
   ! initialise some basic code variables
@@ -627,6 +640,9 @@ program splash
      call select_kernel(ikernel)
   endif
 
+  ! set geometry defaults
+  call initialise_coord_transforms
+
   if (doconvert) then
 
      !
@@ -644,6 +660,15 @@ program splash
         call get_data(1,ihavereadfilenames,firsttime=.true.)
      endif
 
+     ! read tabulated colour table, if necessary
+     if (abs(icolours)==icustom) then
+        call read_asciifile(rgbfile,ncoltable,rgbtable,ierr)
+        if (ierr /= 0 .or. ncoltable <= 0) then
+           print "(a)",'ERROR: could not read colours from '//trim(rgbfile)
+        else
+           print "(a,i3,a)",'read ',ncoltable,' colours from '//trim(rgbfile)
+        endif
+     endif
      !
      ! setup kernel table for fast column density plots in 3D
      !
@@ -742,7 +767,7 @@ subroutine print_header
 20 format(/,  &
    '  ( B | y ) ( D | a | n | i | e | l ) ( P | r | i | c | e )',/)
 
- print "(a)",'  ( '//trim(version)//' Copyright (C) 2005-2017 )'
+ print "(a)",'  ( '//trim(version)//' Copyright (C) 2005-2019 )'
  print 30
 30 format(/,    &
    ' * SPLASH comes with ABSOLUTELY NO WARRANTY.',/, &
