@@ -15,7 +15,7 @@
 !  a) You must cause the modified files to carry prominent notices
 !     stating that you changed the files and the date of any change.
 !
-!  Copyright (C) 2005-2017 Daniel Price. All rights reserved.
+!  Copyright (C) 2005-2018 Daniel Price. All rights reserved.
 !  Contact: daniel.price@monash.edu
 !
 !-----------------------------------------------------------------
@@ -33,6 +33,44 @@ module settings_units
  public :: set_units,read_unitsfile,write_unitsfile,defaults_set_units
  public :: get_nearest_length_unit,get_nearest_time_unit
 
+ integer, parameter :: nx = 7
+ real(doub_prec), parameter :: unit_length(nx) = &
+    (/1.d0,    &
+      1.d5,    &
+      6.96d10, &
+      1.496d13,&
+      3.086d18,&
+      3.086d21,&
+      3.086d24/)
+
+ character(len=*), parameter :: unit_labels_length(nx) = &
+    (/' [cm]     ',&
+      ' [km]     ',&
+      ' [R_{Sun}]',&
+      ' [au]     ',&
+      ' [pc]     ',&
+      ' [kpc]    ',&
+      ' [Mpc]    '/)
+
+ integer, parameter :: nt = 7
+ real(doub_prec), parameter :: unit_time(nt) = &
+    (/1.d-3,        &
+      1.d0,         &
+      3.6d3,        &
+      8.64d4,       &
+      3.15568926d7, &
+      3.15568926d13,&
+      3.15568926d16/)
+
+ character(len=*), parameter :: unit_labels_time(nt) = &
+    (/' ms  ',&
+      ' s   ',&
+      ' hrs ',&
+      ' days',&
+      ' yrs ',&
+      ' Myr ',&
+      ' Gyr '/)
+
  private
 
 contains
@@ -42,7 +80,6 @@ contains
 !
 !---------------------------------------------------------------
 subroutine defaults_set_units
- implicit none
 
  units(:) = 1.0
  unitzintegration = 1.0
@@ -62,27 +99,8 @@ subroutine get_nearest_length_unit(udist,unit,unitlabel)
  real(doub_prec),  intent(in)  :: udist
  real(doub_prec),  intent(out) :: unit
  character(len=*), intent(out) :: unitlabel
- integer, parameter :: nu = 7
- real(doub_prec), parameter :: unit_length(nu) = &
-    (/1.d0,    &
-      1.d5,    &
-      6.96d10, &
-      1.496d13,&
-      3.086d18,&
-      3.086d21,&
-      3.086d24/)
 
- character(len=*), parameter :: unit_labels(nu) = &
-    (/' [cm]     ',&
-      ' [km]     ',&
-      ' [R_{Sun}]',&
-      ' [au]     ',&
-      ' [pc]     ',&
-      ' [kpc]    ',&
-      ' [Mpc]    '/)
-
-
- call get_nearest_unit(nu,unit_length,unit_labels,udist,unit,unitlabel)
+ call get_nearest_unit(nx,unit_length,unit_labels_length,udist,unit,unitlabel)
 
 end subroutine get_nearest_length_unit
 
@@ -95,26 +113,8 @@ subroutine get_nearest_time_unit(utime,unit,unitlabel)
  real(doub_prec),  intent(in)  :: utime
  real(doub_prec),  intent(out) :: unit
  character(len=*), intent(out) :: unitlabel
- integer, parameter :: nu = 7
- real(doub_prec), parameter :: units_time(nu) = &
-    (/1.d-3,        &
-      1.d0,         &
-      3.6d3,        &
-      8.64d4,       &
-      3.15568926d7, &
-      3.15568926d13,&
-      3.15568926d16/)
 
- character(len=*), parameter :: unit_labels(nu) = &
-    (/' ms  ',&
-      ' s   ',&
-      ' hrs ',&
-      ' days',&
-      ' yrs ',&
-      ' Myr ',&
-      ' Gyr '/)
-
- call get_nearest_unit(nu,units_time,unit_labels,utime,unit,unitlabel)
+ call get_nearest_unit(nt,unit_time,unit_labels_time,utime,unit,unitlabel)
 
 end subroutine get_nearest_time_unit
 
@@ -151,14 +151,16 @@ end subroutine get_nearest_unit
 !
 !-------------------------------------------------------
 subroutine set_units(ncolumns,numplot,UnitsHaveChanged)
-  use prompting, only:prompt
-  use labels, only:label,ix,ih,iamvec,labelvec
-  use settings_data, only:ndim,ndimV
-  implicit none
+  use prompting,     only:prompt
+  use labels,        only:label,ix,ih,iamvec,labelvec,headertags
+  use settings_data, only:ndim,ndimV,ivegotdata
+  use particle_data, only:headervals,maxstep
+  use asciiutils,    only:match_tag
   integer, intent(in) :: ncolumns,numplot
   logical, intent(out) :: UnitsHaveChanged
-  integer :: icol
-  real :: unitsprev,dunits
+  integer :: icol,i,ihdr
+  real    :: unitsprev,dunits
+  real(doub_prec) :: udist,utime
   logical :: applytoall
 
   icol = 1
@@ -172,8 +174,24 @@ subroutine set_units(ncolumns,numplot,UnitsHaveChanged)
            print "(a)",' this means that units set here will be re-scalings of these physical values'
         endif
         if (icol.eq.0) then
+           ! give hints for possible time units, if utime is read from data file
+           ihdr = match_tag(headertags,'utime')
+           if (ihdr > 0 .and. maxstep > 0 .and. ivegotdata) then
+              utime = headervals(ihdr,1) ! retrieve from first file in memory
+              do i=1,nt
+                 print "(a,' = ',1pg10.3)",unit_labels_time(i),utime/unit_time(i)
+              enddo
+           endif
            call prompt('enter time units (new=old*units)',units(icol))
         else
+           ! give hints for possible length units, if utime is read from data file
+           ihdr = match_tag(headertags,'udist')
+           if (any(ix==icol) .and. ihdr > 0 .and. maxstep > 0 .and. ivegotdata) then
+              udist = headervals(ihdr,1) ! retrieve from first file in memory
+              do i=1,nx
+                 print "(a,' = ',1pg10.3)",unit_labels_length(i),udist/unit_length(i)
+              enddo
+           endif
            call prompt('enter '//trim(label(icol))//' units (new=old*units)',units(icol))
         endif
         if (abs(units(icol)).gt.tiny(units)) then
@@ -266,7 +284,6 @@ end subroutine set_units
 !
 !-------------------------------------------------------
 subroutine write_unitsfile(unitsfile,ncolumns)
-  implicit none
   character(len=*), intent(in) :: unitsfile
   integer, intent(in) :: ncolumns
   integer :: i,ierr
@@ -300,32 +317,31 @@ end subroutine write_unitsfile
 !-------------------------------------------------------
 subroutine read_unitsfile(unitsfile,ncolumns,ierr,iverbose)
   use settings_data, only:ndim
-  implicit none
   character(len=*), intent(in) :: unitsfile
   integer, intent(in) :: ncolumns
   integer, intent(out) :: ierr
   integer, intent(in), optional :: iverbose
   character(len=2*len(unitslabel)+40) :: line
-  integer :: i,itemp,isemicolon,isemicolon2,isemicolon3
+  integer :: i,itemp,isemicolon,isemicolon2,isemicolon3,isverbose
   logical :: ierrzunits,iexist,verbose
-  
+
   if (present(iverbose)) then
-     verbose= (iverbose.gt.0)
+     isverbose= iverbose
   else
-     verbose = .true.
+     isverbose = 1
   endif
 
   ierr = 0
   ierrzunits = .false.
   inquire(file=unitsfile,exist=iexist)
   if (.not.iexist) then
-     if (verbose) print "(1x,a)",trim(unitsfile)//' not found'
+     if (isverbose > 1) print "(1x,a)",trim(unitsfile)//' not found'
      ierr = 1
      return
   endif
 
   open(unit=78,file=unitsfile,status='old',form='formatted',err=997)
-  if (verbose) print "(a)",' read '//trim(unitsfile)
+  if (isverbose > 0) print "(a)",' read '//trim(unitsfile)
   do i=0,maxplot  ! read all units possibly present in file
 !
 !    read a line from the file
@@ -382,7 +398,7 @@ subroutine read_unitsfile(unitsfile,ncolumns,ierr,iverbose)
   return
 
 997 continue
-  if (verbose) print*,trim(unitsfile),' not found'
+  if (isverbose > 0) print*,trim(unitsfile),' not found'
   ierr = 1
   return
 998 continue
